﻿using ProjectMotorboat.Drownings;
using ProjectMotorboat.Entities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectMotorboat
{
    public partial class FormBoatConfig : Form
    {
        private DrawningBoat? _boat;

        private event BoatDelegate? BoatDelegate;
        public FormBoatConfig()
        {
            InitializeComponent();
            panelRed.MouseDown += Panel_MouseDown;
            panelGreen.MouseDown += Panel_MouseDown;
            panelBlue.MouseDown += Panel_MouseDown;
            panelYellow.MouseDown += Panel_MouseDown;
            panelWhite.MouseDown += Panel_MouseDown;
            panelGray.MouseDown += Panel_MouseDown;
            panelBlack.MouseDown += Panel_MouseDown;
            panelPurple.MouseDown += Panel_MouseDown;

            buttonCancel.Click += (sender, e) => Close();
        }

        public void AddEvent(BoatDelegate boatDelegate)
        {
            BoatDelegate += boatDelegate;
        }

        private void DrawObject()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _boat?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
            _boat?.SetPosition(15, 15);
            _boat?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }

        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
        }

        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text)?.ToString())
            {
                case "labelSimpleObject":
                    _boat = new DrawningBoat((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                    break;
                case "labelModifiedObject":
                    _boat = new DrawningMotorboat((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White,
                    Color.Black, checkBoxMotor.Checked, checkBoxGlass.Checked);
                    break;
            }
            DrawObject();
        }

        private void Panel_MouseDown(object? sender, MouseEventArgs e)
        {
            // TODO отправка цвета в Drag&Drop
            (sender as Control)?.DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);

        }
        // Логика смены цветов: основного и дополнительного (для продвинутого объекта)
        private void LabelBodyColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }


        private void LabelBodyColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_boat != null)
            {
                if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
                {
                    _boat.EntityBoat.SetBodyColor((Color)e.Data.GetData(typeof(Color)));

                }
                DrawObject();
            }
        }
        private void LabelAdditionalColor_DragEnter(object sender, DragEventArgs e)
        {
            if (_boat != null && _boat is DrawningMotorboat)
            {
                if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
                {
                    e.Effect = DragDropEffects.Copy;
                }
                else
                {
                    e.Effect = DragDropEffects.None;
                }
            }
        }

        private void LabelAdditionalColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_boat != null && _boat.EntityBoat is EntityMotorboat _battleship)
            {
                if (e.Data.GetDataPresent(typeof(Color)))
                {
                    _battleship.SetAdditionalColor((Color)e.Data.GetData(typeof(Color)));

                }
                DrawObject();
            }
        }

        private void ButtonAdd_Click(object sender, EventArgs e)
        {

            if (_boat != null)
            {
                BoatDelegate?.Invoke(_boat);
                Close();
            }

        }
    }

}
